/**********************************************************************
*  2006 Microchip Technology Inc.
*
* FileName:        main.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       dsPIC24fj64ga002
* Compiler:        MPLAB C30 v2.02.00 or higher
*
* Running with Internal RC x 4PLL or 32Mhz -> 16MIPS
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author  Thomas Kibalo           Date   1/30/08   Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Using PICKIT2 with new version of MPLAB 8.01
* testing pic24 for robot sensor board servo timing generation using OC modules
* added capability to ping Parallel device--uses RB5 and IC1--works!
* 11/23 issue with inches/measurement not being long CPU crashed--raised exception
* 11/24 made measurement and inches into longs excpetion disappeared
* 11/24 added ping1 for IC1
* 11/24 added servo initialization and servo functions servo on/off servo set
*11/24 copied RadioShack PICKIT_PIC24Fver3 and made BlueRoguever0
*5/8/10  intialied rework to do freq measurement and accelerometer capture
*5//8/10 modify for experimenter added IC2 for second measurments
* IC1 is X  pin 8 of I/O EXP
* IC2 is Y  pin 7 of I/O EXP
*use  algorthimn a(g) = (pulse width/period -.5)/12.5%
* use asin to calculate titl angle
*works Cx,y displays x,y counts from IC1,IC2 capture
	   Gx,y displays x,y g force
	   Ax,y dispays x,y angles relative to earth
*add averaging


***************************************************/
// Include Header File
//***************************************************/

#include "p24fj64ga002.h"
#include "lcd.h"
#include "delay.h"
#include <math.h>
#include <stdio.h>
//***************************************************/
//
//***************************************************/
// Define Constants Variables
//***************************************************/
#define center 0x0BB8 //1.5msec
#define left   0x07D0 // 1 msec
#define right  0x0FA0 //2 msec
//***********************************
#define IC1_RISE IC1CONbits.ICM= 3
#define IC1_FALL IC1CONbits.ICM=2
#define IC1_OFF  IC1CONbits.ICM=0
#define IC1_EV16 IC1CONbits.ICM=5
//**********************************
#define IC2_RISE IC2CONbits.ICM= 3
#define IC2_FALL IC2CONbits.ICM=2
#define IC2_OFF  IC2CONbits.ICM=0
#define IC1_EV16 IC1CONbits.ICM=5
//*********************************
#define periodcnt  20000  //calc count for 100 hz waveform

volatile unsigned char capture_state1 =0;
volatile int high_time1 =0;
volatile int low_time1=0;
volatile int pw1 =0;
volatile int flagrdy1 =0;
volatile unsigned char capture_state2 =0;
volatile int high_time2 =0;
volatile int low_time2=0;
volatile int flagrdy2 =0;
volatile int pw2 =0;

double anglex = 0;
double angley =0;
double temp =0;
double temp1 =0;
long pulseavgx[] ={ 0,0,0,0};
long pulseavgy[] ={0,0,0,0};
int xindex =0;
int yindex =0;
int averagex =0;
int averagey =0;

int mypw1 =0;
int mypw2 =0;
double ax;
double ay;
char bcd10000 =0;
char bcd1000=0;
char bcd100 =0;
char bcdtens =0;
char bcdunits =0; 
//int result;
const char tilt[] =     "Ax,y          ";
 const char xyvalue[] = "Cx,y          ";
const char axyvalue[] = "Gx,y          ";

char angledata [10];
void binary_to_ASCIIconvert( int n) {
bcd10000 =0x00;
bcd1000 = 0x00;
bcd100 =0x00;
bcdtens =0x00;
bcdunits =0x00;
while (n>=10000) {
n = n-10000;
bcd10000 =bcd10000+1;
}
while (n>=1000) {
n = n-1000;
bcd1000 =bcd1000+1;
}
while (n>=100) {
n = n-100;
bcd100 =bcd100+1;
}
while (n>=10) {
n = n-10;
bcdtens =bcdtens+1;
}
while (n>0) {
n = n-1;
bcdunits =bcdunits+1;
}
bcd10000 = bcd10000+0x30;
bcd1000 =bcd1000+0x30;
bcd100 =bcd100+0x30;
bcdtens =bcdtens+0x30;
bcdunits =bcdunits+0x30;
}




/*************** PING ROUNTINE ***********************************
// interrupt service routine for ping1 IC1
*****************************************************************/
void __attribute__((interrupt, no_auto_psv)) _IC1Interrupt(void){
//capture the echo return from PING using interrupt state machine
	
	switch (capture_state1)
	{
	case 0:  //received leading edge
	flagrdy1 =0;
		capture_state1 =1;
		high_time1 =IC1BUF;	//retrieve capture time to rising edge of echo
		IC1_FALL;			//set IC1 to capture falling edge of echo
		break;
	case 1: // received falling edge
		low_time1=IC1BUF	;	//retreive capture time to falling edge of echo
		IC1_OFF;			//turn off IC1
		_IC1IE=0;			//disable interrupt
		capture_state1=0;	//set state to completion
	pw1 = low_time1-high_time1;  //do pulse calcuation
	TMR3=0;
	T3CONbits.TON =0; //turn off timer
	flagrdy1 =1;
		break;
	default:
		capture_state1 =0;	//default 
		IC1_OFF;
		TMR3=0;
	T3CONbits.TON =0; //turn on timer
		_IC1IE=0;
		flagrdy1 =0;
		break;
	}
				
	_IC1IF =0;		//on exit clear IC1 interrupt flag
		
		
	}
	void __attribute__((interrupt, no_auto_psv)) _IC2Interrupt(void){
//capture the echo return from PING using interrupt state machine
	
	switch (capture_state2)
	{
	case 0:  //received leading edge
	flagrdy2 =0;
		capture_state2 =1;
		high_time2 =IC2BUF;	//retrieve capture time to rising edge of echo
		IC2_FALL;			//set IC1 to capture falling edge of echo
		break;
	case 1: // received falling edge
		low_time2=IC2BUF	;	//retreive capture time to falling edge of echo
		IC2_OFF;			//turn off IC1
		_IC2IE=0;			//disable interrupt
		capture_state2=0;	//set state to completion
	pw2 = low_time2-high_time2;  //do pulse calcuation
	TMR2=0;
	T2CONbits.TON =0; //turn off timer
	flagrdy2 =1;
		break;
	default:
		capture_state2 =0;	//default 
		IC2_OFF;
		T2CONbits.TON =0; //turn off timer
		TMR2=0;
		_IC2IE=0;
		flagrdy2 =0;
		break;
	}
				
	_IC2IF =0;		//on exit clear IC1 interrupt flag
		
		
	}

void capture_IC1(void) { //uses TMR3
	IC1CON =0x0000;		//enable IC1 TMR3
		PR3 =0xffff;		//initialize timer3
 		T3CON = 0x0010 ;    //1:8 prescale
 	
		TMR3 = 0x0000;  //clear timer3
		_IC1IE =0;
		IC1_OFF;		//turn off IC1
	   _IC1IF =0;
	   flagrdy1 =0;
		capture_state1 =0; //initialize state machine for interrupt
		T3CONbits.TON =1; //turn on timer
		IC1_RISE;		//capture rise
		_IC1IF =0;
		_IC1IE =1;		//enable interrupt	
	
	
}		

void capture_IC2(void) {	//uses TMR2
		IC2CON =0x0080;		//enable IC2 timer2
		PR2 =0xffff;		//initialize timer3
 		T2CON = 0x0010 ;    //1:8 prescale
 	
		TMR2 = 0x0000;  //clear timer3
		_IC2IE =0;
		IC2_OFF;		//turn off IC1
	   _IC2IF =0;
	   flagrdy2 =0;
		capture_state2 =0; //initialize state machine for interrupt
		T2CONbits.TON =1; //turn on timer
		IC2_RISE;		//capture rise
		_IC2IF =0;
		_IC2IE =1;		//enable interrupt	
	
	
	
	
}		
	

	
//***************************************************/
int main (void)
{
	CLKDIVbits.RCDIV = 0; //no clk div stay at 16MIPS

__builtin_write_OSCCONL(OSCCON & 0xbf) ; //clear bit 6
//input
//	RPINR18bits.U1RXR = 9;	// Make Pin RP9 U1RX
	RPINR7bits.IC1R =13;		// make RP13 = OC1 pin 8 EXP CONN
	RPINR7bits.IC2R =12;		// make RP12 = OC2 pin 7 EXP CONN
//output

	
__builtin_write_OSCCONL(OSCCON | 0x40) ; //set bit 6


 LCD_Initialize();
 clear_display();
 write_string_LCD ( (char*)xyvalue);
  position_cursor(row2);
 write_string_LCD( (char*) axyvalue);
  position_cursor(row3);
write_string_LCD( (char*) tilt);

	TRISBbits.TRISB13=1;
		TRISBbits.TRISB12=1;



while(1) {
	
//int pulseavgx[] ={ 0,0,0,0};
//int pulseavgy[] =[0,0,0,0};
//int xindex =0;
//int yindex =0;
//int averagex =0;
//int averagey =0;	
//	

flagrdy1=0;
capture_IC1();
while( flagrdy1==0);
flagrdy1=0;
mypw1 =pw1; //x
pulseavgx[xindex] =( long)mypw1;



flagrdy2 =0;
capture_IC2();
while( flagrdy2==0);
flagrdy2=0;
mypw2 =pw2; //y
pulseavgy[yindex] =(long) mypw2;
//handle queues
xindex++;
if (xindex==4) xindex =0;
yindex++;
if (yindex==4) yindex =0;
//do averaging
averagex = (int)((pulseavgx[0] +  pulseavgx[1] +  pulseavgx[2] +  pulseavgx[3])/4 );
averagey = (int)((pulseavgy[0] +  pulseavgy[1] +  pulseavgy[2] +  pulseavgy[3])/4 );
mypw1 = (int)averagex;
mypw2 =(int) averagey;
binary_to_ASCIIconvert(mypw1);
 position_cursor(row1+5);
 write_character_LCD(bcd10000);
 write_character_LCD(bcd1000);
 write_character_LCD(bcd100);
 write_character_LCD(bcdtens);
 write_character_LCD(bcdunits);
 binary_to_ASCIIconvert(mypw2);

 write_character_LCD(',');
 write_character_LCD(bcd10000);
 write_character_LCD(bcd1000);
 write_character_LCD(bcd100);
 write_character_LCD(bcdtens);
 write_character_LCD(bcdunits);
temp = (double) mypw1;   //x
temp1 = (double) mypw2;  //y
anglex = (temp/periodcnt-.5)*8 ;


sprintf (angledata,"%1.2f",anglex);
position_cursor(row2+5);
write_string_LCD(angledata);
 write_character_LCD(',');
angley = (temp1/periodcnt-.5)*8 ;
sprintf (angledata,"%1.2f",angley);
write_string_LCD(angledata);
//lets do titlt on x
if ( anglex >=1) { anglex = 90.0; goto out; }
if (anglex <= -1) {anglex =-90.0 ; goto out; }
anglex = asin(anglex)*( 90/ (3.141593/2));
out: sprintf (angledata,"%2.1f",anglex);
position_cursor(row3+5);
write_string_LCD(angledata);
 write_character_LCD(',');
//lets do titlt on y
if ( angley >=1) { angley = 90.0; goto out1; }
if (angley <= -1) {angley =-90.0 ; goto out1; }
angley = asin(angley)*( 90/ (3.141593/2));
out1: sprintf (angledata,"%2.1f",angley);

write_string_LCD(angledata);
	delay();
	delay();	
};		

}
//end of code
