/**********************************************************************
*  2006 Microchip Technology Inc.
*
* FileName:        main.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       dsPIC24fj64ga002
* Compiler:        MPLAB C30 v2.02.00 or higher
*
* Running with Internal RC x 4PLL or 32Mhz -> 16MIPS
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author  Thomas Kibalo           Date   1/30/08   Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Using PICKIT2 with new version of MPLAB 8.01
* testing pic24 for robot sensor board servo timing generation using OC modules
* added capability to ping Parallel device--uses RB5 and IC1--works!
* 11/23 issue with inches/measurement not being long CPU crashed--raised exception
* 11/24 made measurement and inches into longs excpetion disappeared
* 11/24 added ping1 for IC1
* 11/24 added servo initialization and servo functions servo on/off servo set
*11/24 copied RadioShack PICKIT_PIC24Fver3 and made BlueRoguever0 
*05/8/10 coverted to ping just reading sensor for Experimenter
left IC1 RP5 pin 3 along with RB5 I/O for ping snesor
assigend OC1 RP12 pin 7 for servo #1
assigend OC2 RP13 pin 8 for servo #2

***************************************************/
// Include Header File
//***************************************************/

#include "p24fj64ga002.h"
#include "lcd.h"
#include "adc.h"
#include "delay.h"

//***************************************************/
//
//***************************************************/
// Define Constants Variables
//***************************************************/
#define center 0x0BB8 //1.5msec
#define left   0x07D0 // 1 msec
#define right  0x0FA0 //2 msec

//PING interface uses IC1 

#define IC1_RISE IC1CONbits.ICM= 3
#define IC1_FALL IC1CONbits.ICM=2
#define IC1_OFF  IC1CONbits.ICM=0
#define RB5_OUT TRISBbits.TRISB5=0;
#define RB5_IN  TRISBbits.TRISB5=1;
#define out_Pulse_high LATBbits.LATB5=1;
#define out_Pulse_low  LATBbits.LATB5 =0;

#define OC2_ON  OC2CONbits.OCM =6
#define OC2_OFF OC2CONbits.OCM =0
#define OC1_ON  OC1CONbits.OCM =6
#define OC1_OFF OC1CONbits.OCM =0
#define LF	0x0A
#define CR	0x0D
#define SPACE 0x20
#define iend 500
#define jend 1000

static unsigned char capture_state =0;
static int high_time =0;
static int low_time=0;
static long measure =0;
static long inches;
char bcd100 =0;
char bcdtens =0;
char bcdunits =0; 
//int result;
const char banner[] = "Ping Demo";
const char title[] = "target distance";
const char distance[] = "    inches";

void binary_to_ASCIIconvert( int n) {
bcd100 =0x00;
bcdtens =0x00;
bcdunits =0x00;
while (n>=100) {
n = n-100;
bcd100 =bcd100+1;
}
while (n>=10) {
n = n-10;
bcdtens =bcdtens+1;
}
while (n>0) {
n = n-1;
bcdunits =bcdunits+1;
}
bcd100 =bcd100+0x30;
bcdtens =bcdtens+0x30;
bcdunits =bcdunits+0x30;
}



/*********************************************************************************
* servo functions ****************************************************************
*********************************************************************************/

void servo_init(void) { //initializes servo on OC1 and OC2 to center positions uses Timer2
	PR2 =0x9C40;    //20 msec @ 16 MIP
	TMR2 =0x0000;
	T2CON = 0x0000;  // no 32 bit mod presecal no prescale fosc/2 clock ( 16MIPS-> 2 MIPS or 500 nanosec
	T2CONbits.TCKPS= 0x01;  //prescaler 1:8
	T2CONbits.TON =0; //turn it off
	OC1RS = center;	//1.5msec
	OC1CON = 0x0006;  // TMR2, PWM
	OC2RS =center;	// 1.5 msec
	OC2CON = 0x0006;  // TMR2, PWM
	OC1_ON;
	OC2_ON;
	T2CONbits.TON =1; //turn it on
	
	}

void servo1_off( void) {
	OC1_OFF;
}
void servo1_on( void) {
	OC1_ON;
}
void servo2_off( void) {
	OC2_OFF	;
}
void servo2_on( void) {
	OC2_ON;
}		

void set_servo1( int set) {
	OC1RS =set;
}
void set_servo2( int set) {
	OC2RS =set;
}
/*************** PING ROUNTINE ***********************************
// interrupt service routine for ping1 IC1
*****************************************************************/
void __attribute__((interrupt, no_auto_psv)) _IC1Interrupt(void){
//capture the echo return from PING using interrupt state machine
	
	switch (capture_state)
	{
	case 0:  //received leading edge
		capture_state =1;
		high_time =IC1BUF;	//retrieve capture time to rising edge of echo
		IC1_FALL;			//set IC1 to capture falling edge of echo
		break;
	case 1: // received falling edge
		low_time=IC1BUF	;	//retreive capture time to falling edge of echo
		IC1_OFF;			//turn off IC1
		_IC1IE=0;			//disable interrupt
		capture_state=2;	//set state to completion
	
		break;
	default:
		capture_state =0;	//default 
		IC1_OFF;
			_IC1IE=0;
		break;
	}
				
	_IC1IF =0;		//on exit clear IC1 interrupt flag
		
		
	}
// ping1 service for RADIO SHACK PING device --used interrupt state machine and IC1
	
void ping1(void) { //pings on IC1---only one ICX is enabled at at time
	//all pings use global int low/high and global long uses Timer3
		IC1CON =0x0000;		//enable IC1
		PR3 =0xffff;		//initialize timer3
 		T3CON = 0x0010 ;    //1:8 prescale
		TMR3 = 0x0000;  //clear timer3
		_IC1IE =0;
		IC1_OFF;		//turn off IC1
	   _IC1IF =0;
		capture_state =0; //initialize state machine for interrupt
		RB5_OUT;		//set RB5 as output
		out_Pulse_low;	//RB5 pulse out for trigger
		delay_5us();	//5 usec pulse
		out_Pulse_high;
		delay_5us();
		out_Pulse_low;
		delay_5us();
		RB5_IN;			// set RB5 as input
		delay_100us();	//hold off
		delay_100us();
		delay_100us();
		delay_100us();
		T3CONbits.TON =1; //turn on timer
		IC1_RISE;		//capture rise
		_IC1IF =0;
		_IC1IE =1;		//enable interrupt
	while (capture_state !=2); //wait for state machine shows completion

 	T3CONbits.TON =0;  //turn off timer
	
		
}		
	

	
//***************************************************/
int main (void)
{
	CLKDIVbits.RCDIV = 0; //no clk div stay at 16MIPS

__builtin_write_OSCCONL(OSCCON & 0xbf) ; //clear bit 6
//input
//	RPINR18bits.U1RXR = 9;	// Make Pin RP9 U1RX
	RPINR7bits.IC1R =5;		// make RP5 = OC1
//output
//	RPOR4bits.RP8R = 3;		// Make Pin RP8 U1TX
	RPOR6bits.RP12R =18; // OC1 becomes RP12  pin 7
	RPOR6bits.RP13R =19; // OC2 becomes RP13 pin 8
	
__builtin_write_OSCCONL(OSCCON | 0x40) ; //set bit 6

// _ODB10=1;	//set servos out to open drain---need +5V out through LM324 op amp
// _ODB11=1;

_ODB12=1;	//set servos out to open drain---need +5V out through LM324 op amp
 _ODB13=1; // pin 8 is OC1 RP13  , pin 7 is OC2 RP12
// LED Peripheral Initialisation
	LATBbits.LATB15 =0;	//initialize to no led
	TRISBbits.TRISB15=0;
LCD_Initialize();
clear_display();
write_string_LCD( (char*) banner);
position_cursor(row2);
write_string_LCD( (char*) title);
position_cursor(row3);
write_string_LCD( (char*)distance);

	servo_init();    ///initializes OC1 and OC2 for center positions of servos
	
	_IC1IP =4;
	//INTCON1bits.NSTDIS =1;
	_IPL =0;



while(1) {
	
	ping1();


//////// take times and make measurement into inches
//////

	measure = (long) (low_time- high_time);
	measure =measure/4 ;  //remove dual path first 2 divde and adjust cout second divdie by 2
   inches =measure/75 ;
   //convert inches to ascii;
     binary_to_ASCIIconvert((int)inches);
position_cursor(row3);
//supress leading zero
if (bcd100 =='0') {
bcd100 =' ';
if (bcdtens =='0') 
{
bcdtens =' ';	
}	
}
write_character_LCD(bcd100);
write_character_LCD(bcdtens);
write_character_LCD(bcdunits);
//	
	PORTBbits.RB15 =1;  //pin 10 of expansion bus -LED
	delay();
	delay();
	delay();

	PORTBbits.RB15 =0; //pin 10 of expansion bus -LED

	delay();
	delay();	
};		

}
//end of code
